import binascii
from collections import defaultdict

def is_hex(s):
    try:
        int(s, 16)
        return True
    except ValueError:
        return False

# Function to process the lines and perform the subtraction for timestamps
def process_timestamps(lines):
    results = []
    for i in range(len(lines) - 1):
        # Extract the 8 hex digits from each line
        hex1 = lines[i][:8]
        hex2 = lines[i + 1][:8]
        
        # Convert hex to integers
        int1 = int(hex1, 16)
        int2 = int(hex2, 16)
        
        # Perform the subtraction
        diff = int2 - int1
        
        # Convert the difference to decimal and hex
        diff_decimal = diff
        diff_hex = format(diff, 'x').upper()
        
        # Format the result string
        result = f"{hex1} - {hex2}   =   {diff_decimal}   =   {diff_hex}"
        results.append(result)
    
    return results

# Function to process the lines and perform the subtraction for increments
def process_increments(lines):
    results = []
    for i in range(len(lines) - 1):
        # Extract the 8 hex digits from each line
        
        hx_check = lines[i][20:28]
        if is_hex(hx_check):
            hex1 = lines[i][20:28]
        else:
            hex1 = "0"
        
        hx_check = lines[i + 1][20:28]
        if is_hex(hx_check):
            hex2 = lines[i + 1][20:28]
        else:
            hex2 = "1"
        
        # Convert hex to integers
        int1 = int(hex1, 16)
        int2 = int(hex2, 16)
        
        # Perform the subtraction
        diff = int2 - int1
        
        # Convert the difference to decimal and hex
        diff_decimal = diff
        diff_hex = format(diff, 'x').upper()
        
        # Format the result string
        result = f"{hex1} - {hex2}   =   {diff_decimal}   =   {diff_hex}"
        results.append(result)
    
    return results

# Function to calculate summary statistics
def calculate_summary(lines, x):
    summary_results = []
    
    # Difference between the 10th frame and the 5th from the end
    if len(lines) >= 15:
        hex_10th_timestamp = lines[9][:8]
        hex_5th_from_end_timestamp = lines[-5][:8]
        
        hex_10th_increment = lines[9][20:28]
        hex_5th_from_end_increment = lines[-5][20:28]
        
        timestamp_diff_decimal = int(hex_5th_from_end_timestamp, 16) - int(hex_10th_timestamp, 16)
        increment_diff_decimal = int(hex_5th_from_end_increment, 16) - int(hex_10th_increment, 16)
        
        summary_results.append(f"Difference between 10th frame and 5th from end (timestamp): {timestamp_diff_decimal}")
        summary_results.append(f"Difference between 10th frame and 5th from end (increment): {increment_diff_decimal}")
    
    # Check if increments go sequentially and count mismatches
    mismatches_count = 0
    for i in range(len(lines) - 1):
        increment_current = int(lines[i][20:28], 16)
        increment_next = int(lines[i + 1][20:28], 16)
        
        if increment_next != increment_current + 1:
            mismatches_count += 1
    
    summary_results.append(f"Number of mismatches in increments: {mismatches_count}")
    
    # Calculate average difference in timestamps
    total_diff_timestamps = sum(int(lines[i + 1][:8], 16) - int(lines[i][:8], 16) for i in range(len(lines) - 1))
    avg_diff_timestamps = total_diff_timestamps / (len(lines) - 1)
    
    summary_results.append(f"Average difference in timestamps: {avg_diff_timestamps}")
    
    # Count cases where timestamp difference is greater than x and calculate percentage
    count_greater_than_x = sum(1 for i in range(len(lines) - 1) if (int(lines[i + 1][:8], 16) - int(lines[i][:8], 16)) > x)
    percentage_greater_than_x = (count_greater_than_x / (len(lines) - 1)) * 100
    
    summary_results.append(f"Number of cases where timestamp difference is greater than {x}: {count_greater_than_x}")
    summary_results.append(f"Percentage of cases where timestamp difference is greater than {x}: {percentage_greater_than_x:.2f}%")
    
    # Total number of frames received
    total_frames_received = len(lines)
    
    summary_results.append(f"Total number of frames received: {total_frames_received}")
    
    return summary_results

# Main function to handle file reading and processing
def main():
    input_file = 'data.txt'
    path_for_files = 'D://YOUR//PATH//'
    input_file = path_for_files + input_file
    malformed_packet = 0

    try:
        with open(input_file, 'r') as file:
            lines = file.readlines()
    except FileNotFoundError:
        print("Input file not found")
        return

    # Group lines by device ID
    device_logs = defaultdict(list)
    for line in lines:
        if line.strip() == "":
            continue
        
        # if len(line.strip().split(';')[0]) == 34 and len(line.strip().split(';')[1]) == 9 and ';' in line:
        if ';' in line and len(line.strip().split(';')[0]) == 34 and len(line.strip().split(';')[1]) == 9:
            try:
                data, device_id = line.strip().split(';')
                device_logs[device_id].append(data)
            except ValueError:
                print(f"Malformed line: {line.strip()}")
                malformed_packet += 1
        else:
            print(f"Malformed line: {line.strip()}")
            malformed_packet += 1

    # Process each device's logs separately
    # x_value = 1000  # Example value for x
    # x_value = 10 # 10ms
    # x_value = 11 # 
    x_value = 9 
    
    for device_id, device_lines in device_logs.items():
        # Process timestamps and increments
        timestamp_results = process_timestamps(device_lines)
        increment_results = process_increments(device_lines)
        
        # Write timestamp results to file
        with open(f'{path_for_files}results_timestamp_difference{device_id}.txt', 'w') as file:
            file.write(f'\tFor device ID: {device_id}\n')
            for result in timestamp_results:
                file.write(result + '\n')
        
        # Write increment results to file
        with open(f'{path_for_files}results_counter_difference_{device_id}.txt', 'w') as file:
            file.write(f'\tFor device ID: {device_id}\n')
            for result in increment_results:
                file.write(result + '\n')
        
        # Calculate summary statistics
        summary_results = calculate_summary(device_lines, x_value)
        
        # Write summary results to file
        with open(f'{path_for_files}results_summary_{device_id}.txt', 'w') as file:
            file.write(f'\tFor device ID: {device_id}\n')
            for result in summary_results:
                file.write(result + '\n')

    print("Processing complete. Results have been written to respective files for each device.")
    print(f"Number of malformed packets: {malformed_packet}")

if __name__ == "__main__":
    main()
